﻿
namespace MyPictureViewer.Assets
{
   using System;
   using System.Collections.Generic;
   using System.ComponentModel;
   using System.Threading;

   /// <summary>
   /// Load photos from website
   /// </summary>
   public class PhotoProvider : IPhotoProvider
   {
      public event EventHandler DataAvailable;
      private readonly SynchronizationContext context;

      public IList<PhotoInfo> Photos { get; set; }

      public PhotoProvider()
      {
         this.context = SynchronizationContext.Current;
      }

      /// <summary>
      /// Load photos
      /// </summary>
      public void LoadPhotos()
      {
         if (DesignerProperties.IsInDesignTool) return;

         //Load photos in a secondary thread to speed up startup
         ThreadPool.QueueUserWorkItem((o) =>
                                         {
                                           
                                            this.Photos = new List<PhotoInfo>();
                                            Random rnd = new Random();
                                            for (int i = 1; i < 12; i++)
                                            {
                                               PhotoInfo photoInfo = new PhotoInfo()
                                                                        {
                                                                           Comment = "Commento " + i,
                                                                           ImageUri = string.Format("http://localhost:1366/Images/Photo ({0}).jpg", i),
                                                                        };
                                               this.Photos.Add(photoInfo);
                                            }

                                            this.OnDataAvailable();
                                         });
      }

      /// <summary>
      /// Raises DataAvailable event
      /// </summary>
      private void OnDataAvailable()
      {
         if (this.DataAvailable != null)
         {
            this.context.Post((o) => this.DataAvailable(this, EventArgs.Empty), null);
         }
      }
   }
}
