﻿
namespace MyPictureViewer.ViewModels
{
   using System;
   using System.Collections.ObjectModel;
   using GalaSoft.MvvmLight;
   using GalaSoft.MvvmLight.Command;
   using MyPictureViewer.Assets;

   public class MainViewModel : ViewModelBase
   {
      private PhotoInfo selectedImage;
      private int index;
      private bool photoAvailable;
      private ObservableCollection<PhotoInfo> photos;

      /// <summary>
      /// Initializes ViewModel associated with MainPage
      /// </summary>
      /// <param name="provider"></param>
      public MainViewModel(IPhotoProvider provider)
      {
         //throw new InvalidOperationException();
         this.NextCommand = new RelayCommand(this.OnNext);
         this.PreviousCommand = new RelayCommand(this.OnPrevious);
         provider.DataAvailable += this.OnDataAvailable;
         provider.LoadPhotos();
      }

      /// <summary>
      /// Previous photo command
      /// </summary>
      public RelayCommand NextCommand { get; private set; }

      /// <summary>
      /// Next Photo command
      /// </summary>
      public RelayCommand PreviousCommand { get; private set; }

      /// <summary>
      /// Available photos
      /// </summary>
      public ObservableCollection<PhotoInfo> Photos
      {
         get
         {
            return this.photos;
         }

         set
         {
            if (value != this.photos)
            {
               this.photos = value;
               this.RaisePropertyChanged("Photos");
            }
         }
      }

      /// <summary>
      /// Currently selected image
      /// </summary>
      public PhotoInfo SelectedImage
      {
         get
         {
            return this.selectedImage;
         }

         set
         {
            if (value != this.selectedImage)
            {
               this.selectedImage = value;
               this.index = this.Photos.IndexOf(value);
               this.RaisePropertyChanged("SelectedImage");
            }
         }
      }

      /// <summary>
      /// Indicates that photos are available
      /// </summary>
      public bool PhotoAvailable
      {
         get
         {
            return this.photoAvailable;
         }

         set
         {
            if (value != this.photoAvailable)
            {
               this.photoAvailable = value;
               this.RaisePropertyChanged("PhotoAvailable");
            }
         }
      }

      /// <summary>
      /// Load photos when providers signals availability
      /// </summary>
      private void OnDataAvailable(object sender, EventArgs e)
      {
         PhotoProvider provider = (PhotoProvider)sender;
         this.PhotoAvailable = true;
         this.Photos = new ObservableCollection<PhotoInfo>(provider.Photos);
      }

      /// <summary>
      /// Selects next photo
      /// </summary>
      private void OnNext()
      {
         this.index = Math.Min(++this.index, this.Photos.Count - 1);
         this.SelectedImage = this.Photos[this.index];
      }

      /// <summary>
      /// Selects previous photo
      /// </summary>
      private void OnPrevious()
      {
         this.index = Math.Max(--this.index, 0);
         this.SelectedImage = this.Photos[this.index];
      }
   }
}
