// Type: System.ServiceModel.DomainServices.Client.EntitySet`1
// Assembly: System.ServiceModel.DomainServices.Client, Version=2.0.5.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35
// Assembly location: C:\Program Files (x86)\Microsoft SDKs\RIA Services\v1.0\Libraries\Silverlight\System.ServiceModel.DomainServices.Client.dll

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;

namespace System.ServiceModel.DomainServices.Client
{
   /// <summary>
   /// Represents a collection of <see cref="T:System.ServiceModel.DomainServices.Client.Entity"/> instances, providing change tracking and other services.
   /// 
   /// </summary>
   /// <typeparam name="TEntity">The type of <see cref="T:System.ServiceModel.DomainServices.Client.Entity"/> this set will contain</typeparam>
   public sealed class EntitySet<TEntity> : EntitySet, IEnumerable<TEntity>, IEnumerable, ICollectionViewFactory
      where TEntity : Entity
   {
      private EventHandler<EntityCollectionChangedEventArgs<TEntity>> EntityAdded;
      private EventHandler<EntityCollectionChangedEventArgs<TEntity>> EntityRemoved;

      #region ICollectionViewFactory Members

      /// <summary>
      /// Returns a custom view for specialized sorting, filtering, grouping, and currency.
      /// 
      /// </summary>
      /// 
      /// <returns>
      /// A custom view for specialized sorting, filtering, grouping, and currency
      /// </returns>
      ICollectionView ICollectionViewFactory.CreateView();

      #endregion

      #region IEnumerable<TEntity> Members

      IEnumerator<TEntity> IEnumerable<TEntity>.GetEnumerator();

      #endregion

      /// <summary>
      /// Creates the storage list for the set.
      /// 
      /// </summary>
      /// 
      /// <returns>
      /// The created storage list instance.
      /// </returns>
      protected override IList CreateList();

      /// <summary>
      /// Creates a new entity.
      /// 
      /// </summary>
      /// 
      /// <returns>
      /// The created entity instance.
      /// </returns>
      protected override Entity CreateEntity();

      /// <summary>
      /// Returns an enumerator
      /// 
      /// </summary>
      /// 
      /// <returns>
      /// The enumerator
      /// </returns>
      public new IEnumerator<TEntity> GetEnumerator();

      /// <summary>
      /// Attaches the specified <see cref="T:System.ServiceModel.DomainServices.Client.Entity"/> to this <see cref="T:System.ServiceModel.DomainServices.Client.EntitySet"/> in an
      ///             unmodified state, also recursively attaching all unattached entities reachable via
      ///             associations.
      /// 
      /// </summary>
      /// <param name="entity">The entity to attach</param>
      public void Attach(TEntity entity);

      /// <summary>
      /// Detaches the <see cref="T:System.ServiceModel.DomainServices.Client.Entity"/> from this <see cref="T:System.ServiceModel.DomainServices.Client.EntitySet"/>. If the entity
      ///             is not in this set, an <see cref="T:System.InvalidOperationException"/> will be thrown. If the entity
      ///             is the root of a compositional hierarchy, all child entities will also be detached.
      /// 
      /// </summary>
      /// <param name="entity">The entity to detach</param>
      public void Detach(TEntity entity);

      /// <summary>
      /// Add the specified entity to the set, also recursively adding all unattached
      ///             entities reachable via associations.
      /// 
      /// </summary>
      /// 
      /// <remarks>
      /// <paramref name="entity"/> needs to be of type <typeparamref name="TEntity"/>, and cannot be a subclass.
      /// </remarks>
      /// <param name="entity">The entity to add</param>
      public void Add(TEntity entity);

      /// <summary>
      /// Removes the specified entity from the set. If the entity is the root
      ///             of a compositional hierarchy, all child entities will also be removed.
      /// 
      /// </summary>
      /// <param name="entity">The entity to remove</param>
      public void Remove(TEntity entity);

      /// <summary>
      /// Method called whenever the set changes.
      /// 
      /// </summary>
      /// <param name="action">The change action</param><param name="affectedObject">For Reset events, this will be a collection of removed entities. For all other
      ///             events, this will be the single affected entity.</param><param name="index">The affected index</param>
      protected override void OnCollectionChanged(NotifyCollectionChangedAction action, object affectedObject, int index);

      /// <summary>
      /// Event raised whenever an <see cref="T:System.ServiceModel.DomainServices.Client.Entity"/> is added to this collection
      /// 
      /// </summary>
      public event EventHandler<EntityCollectionChangedEventArgs<TEntity>> EntityAdded;

      /// <summary>
      /// Event raised whenever an <see cref="T:System.ServiceModel.DomainServices.Client.Entity"/> is removed to this collection
      /// 
      /// </summary>
      public event EventHandler<EntityCollectionChangedEventArgs<TEntity>> EntityRemoved;

      #region Nested type: ListCollectionViewProxy

      /// <summary>
      /// <see cref="T:System.Collections.IList"/> proxy that makes the <see cref="T:System.ServiceModel.DomainServices.Client.EntitySet"/> usable in the default collection views.
      ///             All operations implemented against the proxy are passed through to the source <see cref="T:System.ServiceModel.DomainServices.Client.EntitySet"/>.
      /// 
      /// </summary>
      /// 
      /// <remarks>
      /// This proxy does not support a full set of list operations. However, the subset it does support
      ///             is sufficient for interaction with the ListCollectionView.
      /// 
      /// </remarks>
      /// <typeparam name="T">The entity type of this proxy</typeparam>
      private class ListCollectionViewProxy<T> : IList, ICollection, IEnumerable<T>, IEnumerable,
                                                 INotifyCollectionChanged where T : Entity
      {
         private readonly EntitySet<T> _source;
         private readonly object _syncRoot;
         private NotifyCollectionChangedEventHandler CollectionChanged;
         internal ListCollectionViewProxy(EntitySet<T> source);
         private EntitySet<T> Source { get; }

         #region IEnumerable<T> Members

         public IEnumerator<T> GetEnumerator();

         #endregion

         #region IList Members

         public int Add(object value);
         public void Clear();
         public bool Contains(object value);
         public int IndexOf(object value);
         public void Insert(int index, object value);
         public void Remove(object value);
         public void RemoveAt(int index);
         public void CopyTo(Array array, int index);
         IEnumerator IEnumerable.GetEnumerator();
         public bool IsFixedSize { get; }
         public bool IsReadOnly { get; }
         public object this[int index] { get; set; }
         public int Count { get; }
         public bool IsSynchronized { get; }
         public object SyncRoot { get; }

         #endregion

         #region INotifyCollectionChanged Members

         public event NotifyCollectionChangedEventHandler CollectionChanged;

         #endregion

         private void OnCollectionChanged(NotifyCollectionChangedEventArgs e);
         private void OnSourceCollectionChanged(object sender, NotifyCollectionChangedEventArgs e);
      }

      #endregion
   }
}
