﻿
namespace DemoCommands.Behaviors
{
   using System.Windows;
   using System.Windows.Controls;
   using System.Windows.Interactivity;

   public class TextChangedBehavior : Behavior<TextBox>
   {

      #region Text

      /// <summary>
      /// Text Dependency Property
      /// </summary>
      public static readonly DependencyProperty TextProperty =
          DependencyProperty.Register("Text", typeof(string), typeof(TextChangedBehavior),
              new PropertyMetadata(null, OnTextChanged));

      /// <summary>
      /// Gets or sets the Text property. This dependency property 
      /// indicates ....
      /// </summary>
      public string Text
      {
         get { return (string)GetValue(TextProperty); }
         set { SetValue(TextProperty, value); }
      }

      /// <summary>
      /// Handles changes to the Text property.
      /// </summary>
      private static void OnTextChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
      {
         TextChangedBehavior target = (TextChangedBehavior)d;
         string oldValue = (string)e.OldValue;
         string newValue = target.Text;
         target.OnTextChanged(oldValue, newValue);
      }

      /// <summary>
      /// Provides derived classes an opportunity to handle changes to the Text property.
      /// </summary>
      protected virtual void OnTextChanged(string oldValue, string newValue)
      {
         this.AssociatedObject.Text = newValue;
      }

      #endregion




      protected override void OnAttached()
      {
         base.OnAttached();

         this.AssociatedObject.TextChanged += this.OnTextChanged;
      }

      private void OnTextChanged(object sender, TextChangedEventArgs e)
      {
         this.Text = this.AssociatedObject.Text;
      }

      protected override void OnDetaching()
      {
         base.OnDetaching();

         this.AssociatedObject.TextChanged -= this.OnTextChanged;
      }
   }
}