﻿using System.Windows;
using System.Windows.Input;

namespace DemoCommandsPhone.Behaviors
{
   using System.Windows.Controls.Primitives;

   public static class InvokeOnClickBehavior
   {
      #region Command

      /// <summary>
      /// Command Attached Dependency Property
      /// </summary>
      public static readonly DependencyProperty CommandProperty =
          DependencyProperty.RegisterAttached("Command", typeof(ICommand), typeof(InvokeOnClickBehavior),
              new PropertyMetadata(null, OnCommandChanged));

      /// <summary>
      /// Gets the Command property. This dependency property 
      /// indicates ....
      /// </summary>
      public static ICommand GetCommand(DependencyObject d)
      {
         return (ICommand)d.GetValue(CommandProperty);
      }

      /// <summary>
      /// Sets the Command property. This dependency property 
      /// indicates ....
      /// </summary>
      public static void SetCommand(DependencyObject d, ICommand value)
      {
         d.SetValue(CommandProperty, value);
      }

      /// <summary>
      /// Handles changes to the Command property.
      /// </summary>
      private static void OnCommandChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
      {
         ICommand oldValue = (ICommand)e.OldValue;
         ICommand newValue = (ICommand)d.GetValue(CommandProperty);

         ButtonBase button = d as ButtonBase;
         if (button != null)
         {
            button.Click += (s, arg) =>
                             {
                                newValue.Execute(null);
                             };
         }
      }

      #endregion




   }
}
