﻿
namespace CloseTheWindow.ViewModels
{
   using System.Windows;
   using GalaSoft.MvvmLight;
   using GalaSoft.MvvmLight.Command;
   using GalaSoft.MvvmLight.Messaging;
   using Messages;
   using Services;

   public class ShellViewModel : ViewModelBase
   {
      private readonly IMessenger messenger;
      private readonly IDialogService dialogService;
      private readonly IWindowManager windowManager;
      private string detailInfo;

      public ShellViewModel(IMessenger messenger, IDialogService dialogService, IWindowManager windowManager)
      {
         this.messenger = messenger;
         this.dialogService = dialogService;
         this.windowManager = windowManager;
         this.CloseCommand = new RelayCommand(this.OnClose);
         this.messenger.Register<string>(this, this.OnDetailMessageReceived);
         this.ShowDetailCommand = new RelayCommand(this.OnShowDetail);
      }

      public RelayCommand CloseCommand { get; private set; }

      public RelayCommand ShowDetailCommand { get; private set; }

      private void OnShowDetail()
      {
         this.windowManager.OpenDetailWindow();
      }

      public string DetailInfo
      {
         get
         {
            return this.detailInfo;
         }

         set
         {
            if (value != this.detailInfo)
            {
               this.detailInfo = value;
               this.RaisePropertyChanged("DetailInfo");
            }
         }
      }

      /// <summary>
      /// Executes CloseCommand
      /// </summary>
      private void OnClose()
      {
         MessageBoxResult result = dialogService.ShowDialog("Confirm exit'", "Exit program", MessageBoxButton.YesNo, MessageBoxImage.Question);
         if (result == MessageBoxResult.Yes)
         {
            QuitMessage message = new QuitMessage();
            this.messenger.Send(message);
         }
      }

      private void OnDetailMessageReceived(string detail)
      {
         this.DetailInfo = detail;
      }
   }
}
