﻿
namespace NavigationDemo.Behaviors
{
   using System;
   using System.Windows;
   using System.Windows.Controls;
   using System.Windows.Interactivity;
   using NavigationDemo.Navigation;

   /// <summary>
   /// Binds a ViewModel to page navigation service
   /// </summary>
   public class NavigationSubscriberBehavior : Behavior<Page>
   {
      #region NavigableInstance

      /// <summary>
      /// NavigableInstance Dependency Property
      /// </summary>
      public static readonly DependencyProperty NavigableInstanceProperty =
          DependencyProperty.Register("NavigableInstance", typeof(INavigable), typeof(NavigationSubscriberBehavior),
              new PropertyMetadata(null));

      /// <summary>
      /// Gets or sets the NavigableInstance property. This dependency property 
      /// indicates ....
      /// </summary>
      public INavigable NavigableInstance
      {
         get { return (INavigable)GetValue(NavigableInstanceProperty); }
         set { SetValue(NavigableInstanceProperty, value); }
      }

      #endregion

      /// <summary>
      /// Handles attached event
      /// </summary>
      protected override void OnAttached()
      {
         base.OnAttached();
         this.AssociatedObject.Loaded += this.OnLoaded;
      }

      /// <summary>
      /// Handles Detaching event
      /// </summary>
      protected override void OnDetaching()
      {
         base.OnDetaching();
         this.AssociatedObject.Loaded -= this.OnLoaded;
      }

      /// <summary>
      /// Associates a navigation service to bound viemodel
      /// </summary>
      /// <param name="sender">Event sender</param>
      /// <param name="e">Event argument</param>
      private void OnLoaded(object sender, RoutedEventArgs e)
      {
         if (this.NavigableInstance == null) throw new InvalidOperationException("ViewModel associated with this page is not compatible with this behavior.");
         this.NavigableInstance.NavigationService = new NavigationService(this.AssociatedObject.NavigationService);
      }
   }
}