﻿
namespace NavigationDemo.Navigation
{
   using System;
   using System.Windows.Navigation;

   /// <summary>
   /// Navigation service engine
   /// </summary>
   public class NavigationService : INavigationService
   {
      private readonly System.Windows.Navigation.NavigationService navigationService;

      /// <summary>
      /// Initializes a new instance of the NavigationService class
      /// </summary>
      /// <param name="navigationService">Page navigation service</param>
      public NavigationService(System.Windows.Navigation.NavigationService navigationService)
      {
         if (navigationService == null) throw new ArgumentNullException("navigationService");
         this.navigationService = navigationService;
         this.navigationService.Navigating += this.OnNavigating;
      }

      public event NavigatingCancelEventHandler Navigating;

      /// <summary>
      /// Navigates to provided uri
      /// </summary>
      /// <param name="uri">Uri to navigate</param>
      public void NavigateTo(string uri)
      {
         this.navigationService.Navigate(new Uri(uri, UriKind.RelativeOrAbsolute));
      }

      /// <summary>
      /// Navigates to previous page
      /// </summary>
      public void GoBack()
      {
         if (this.navigationService.CanGoBack) this.navigationService.GoBack();
      }

      /// <summary>
      /// Handles Navigating event
      /// </summary>
      /// <param name="sender">Event sender</param>
      /// <param name="e">Event argument</param>
      private void OnNavigating(object sender, NavigatingCancelEventArgs e)
      {
         if (this.Navigating != null) this.Navigating(this, e);
      }
   }
}
