﻿

namespace NavigationDemo.Services
{
   using NavigationDemo.Assets;
   using Ninject;
   using Ninject.Modules;
   using ViewModels;

   /// <summary>
   /// Initializes IOC container used to bind all viewmodels instances
   /// </summary>
   public class ServiceLocator : NinjectModule, IServiceLocator
   {
      /// <summary>
      /// Initializes a new instance of the ServiceLocator class
      /// </summary>
      public ServiceLocator()
      {
         this.Kernel = new StandardKernel(this);
      }

      /// <summary>
      /// Gets a type from service locator
      /// </summary>
      /// <typeparam name="T">Type to retrieve</typeparam>
      /// <returns>Requested type</returns>
      public T Get<T>()
      {
         return this.Kernel.Get<T>();
      }

      /// <summary>
      /// Defines and initializes all types used by application
      /// </summary>
      public override void Load()
      {
         this.Bind<IApplicationController>().ToConstant(new ApplicationController());
         this.Bind<IDialogService>().To(typeof(DialogService));
         this.Bind<ShellViewModel>().ToSelf();
         this.Bind<LoginViewModel>().ToSelf();
         this.Bind<AdministrationViewModel>().ToSelf();
         this.Bind<HomeContentViewModel>().ToSelf();
         this.Bind<PreSalesViewModel>().ToSelf();
      }
   }
}
