﻿
namespace NavigationDemo.ViewModels
{
   using System;
   using System.Windows;
   using Assets;
   using GalaSoft.MvvmLight.Command;
   using Services;

   /// <summary>
   /// ViewModel associated with Administration view
   /// </summary>
   public class AdministrationViewModel : ApplicationViewModelBase
   {
      private readonly IDialogService dialogService;

      /// <summary>
      /// Initializes a new instance of the AdministrationViewModel class
      /// </summary>
      /// <param name="applicationController">Global application controller</param>
      /// <param name="dialogService">Application Dialog service</param>
      public AdministrationViewModel(IApplicationController applicationController, IDialogService dialogService)
         : base(applicationController)
      {
         if (dialogService == null) throw new ArgumentNullException("dialogService");
         this.dialogService = dialogService;
         this.ShowMessageCommand = new RelayCommand(this.OnShowMessage);
      }

      /// <summary>
      /// Gets NavigateToNews command
      /// </summary>
      public RelayCommand ShowMessageCommand { get; private set; }

      /// <summary>
      /// Executes Show message command
      /// </summary>
      private void OnShowMessage()
      {
         this.dialogService.ShowMessage("This is just a message...", "Admin message", MessageBoxButton.OK);
      }
   }
}
