﻿

namespace NavigationDemo.ViewModels
{
   using Assets;
   using GalaSoft.MvvmLight.Command;

   /// <summary>
   /// Represents the viewmodel associated with Home View
   /// </summary>
   public class HomeViewModel : ApplicationViewModelBase
   {
      public HomeViewModel(IApplicationController applicationController)
         : base(applicationController)
      {
         this.LogoutCommand = new RelayCommand(this.OnLogout);
         this.NavigateToAdminCommand = new RelayCommand(this.OnNavigateToAdmin);
         this.NavigateToPreSalesCommand = new RelayCommand(this.OnNavigateToPreSales);
         this.NavigateToHomeCommand = new RelayCommand(this.OnNavigateToHome);
      }

      /// <summary>
      /// Gets NavigateToHome command
      /// </summary>
      public RelayCommand NavigateToHomeCommand { get; private set; }

      /// <summary>
      /// Gets NavigateToPreSales command
      /// </summary>
      public RelayCommand NavigateToPreSalesCommand { get; private set; }

      /// <summary>
      /// Gets NavigateToAdmin command
      /// </summary>
      public RelayCommand NavigateToAdminCommand { get; private set; }

      /// <summary>
      /// Gets Login command
      /// </summary>
      public RelayCommand LogoutCommand { get; private set; }

      /// <summary>
      /// Handles Logout command
      /// </summary>
      private void OnLogout()
      {
         this.ApplicationController.ShellNavigationService.NavigateTo("Shell");
      }

      /// <summary>
      /// Handles Navigation to Admin view command
      /// </summary>
      private void OnNavigateToAdmin()
      {
         this.ApplicationController.NavigationService.NavigateTo("Admin");
      }

      /// <summary>
      /// Executes NavigateToPreSalesCommand
      /// </summary>
      private void OnNavigateToPreSales()
      {
         this.ApplicationController.NavigationService.NavigateTo("PreSales");
      }

      /// <summary>
      /// Executes NavigateToHomeCommand
      /// </summary>
      private void OnNavigateToHome()
      {
         this.ApplicationController.NavigationService.NavigateTo("Home");
      }

   }
}
