﻿namespace MvvmOnWindowsPhone.Services
{
   using System;
   using System.ComponentModel;
   using System.Windows;
   using GalaSoft.MvvmLight.Messaging;
   using Messages;
   using Microsoft.Phone.Controls;

   public class NavigationCenter : INavigator
   {
      private readonly PhoneApplicationFrame frame;
      private readonly IMessenger messenger;
      private PhoneApplicationPage activePage;
      private bool isResuming;

      public NavigationCenter(PhoneApplicationFrame frame, IMessenger messenger)
      {
         this.frame = frame;
         this.messenger = messenger;
         if (!DesignerProperties.IsInDesignTool)
         {
            this.frame.Navigating += this.OnNavigating;
            this.frame.Navigated += this.OnNavigated;
         }

         this.messenger.Register<TombstoningMessage>(this, message =>
         {
            if (message.Command == "Resume") this.isResuming = true;
         });
      }


      public void Navigate(string uri)
      {
         this.frame.Navigate(new Uri(uri, UriKind.RelativeOrAbsolute));
      }

      protected virtual void OnNavigating(object sender, System.Windows.Navigation.NavigatingCancelEventArgs e)
      {
         this.activePage = null;
      }

      private void PostPageResumingMessage(PhoneApplicationPage resumedPage)
      {
         if (resumedPage != null)
         {
            TombstoningMessage message = new TombstoningMessage("Resume");
            this.messenger.Send(message);
         }

         this.isResuming = false;
      }

      private void OnPageLoaded(object sender, RoutedEventArgs e)
      {
         this.activePage.Loaded -= this.OnPageLoaded;
         PageLoadedMessage message = new PageLoadedMessage(this.activePage);
         this.messenger.Send(message);

      }

      private void OnNavigated(object sender, System.Windows.Navigation.NavigationEventArgs e)
      {
         this.activePage = e.Content as PhoneApplicationPage;
         if (this.activePage != null)
         {
            this.activePage.Loaded += this.OnPageLoaded;
            if (this.isResuming)
            {
               this.PostPageResumingMessage(this.activePage);
            }
         }
      }
   }
}
