﻿
namespace MvvmOnWindowsPhone.ViewModels
{
   using GalaSoft.MvvmLight;
   using GalaSoft.MvvmLight.Messaging;
   using Messages;

   public abstract class LifeCycleAwareViewModelBase : ViewModelBase
   {

      protected LifeCycleAwareViewModelBase(IMessenger messenger)
      {
         this.Messenger = messenger;
         messenger.Register<TombstoningMessage>(this, this.HandleTombstoning);
         messenger.Register<PageLoadedMessage>(this, this.HandlePageLoaded);
      }

      protected IMessenger Messenger { get; private set; }

      public bool IsResuming { get; set; }

      protected virtual void OnResume()
      {
      }

      protected virtual void OnSuspend()
      {
      }

      protected virtual void OnPageLoaded()
      {
      }

      private void HandlePageLoaded(PageLoadedMessage pageLoadedMessage)
      {
         this.OnPageLoaded();
      }

      private void HandleTombstoning(TombstoningMessage message)
      {
         switch (message.Command)
         {
            case "Suspend":
               this.OnSuspend();
               break;
            case "Resume":
               this.IsResuming = true;
               this.OnResume();
               break;
            default:
               break;
         }
      }
   }
}
