﻿
namespace MvvmOnWindowsPhone.ViewModels
{
   using GalaSoft.MvvmLight.Messaging;
   using Messages;
   using Services;

   public class MainViewModel : LifeCycleAwareViewModelBase
   {
      private readonly ICache cache;
      private string userName;
      private string nickName;

      public MainViewModel(ICache cache, IMessenger messenger)
         : base(messenger)
      {
         this.cache = cache;
      }

      #region Public properties

      public string UserName
      {
         get
         {
            return this.userName;
         }

         set
         {
            if (value != this.userName)
            {
               this.userName = value;
               this.RaisePropertyChanged("UserName");
            }
         }
      }

      public string NickName
      {
         get
         {
            return this.nickName;
         }

         set
         {
            if (value != this.nickName)
            {
               this.nickName = value;
               this.RaisePropertyChanged("NickName");
            }
         }
      }

      #endregion

      protected override void OnSuspend()
      {
         this.cache.ToCache("UserName", this.UserName);
         this.cache.ToCache("NickName", this.NickName);

         string focusedElement = this.GetFocusedElement();
         this.cache.ToCache("FocusedElement", focusedElement);
      }

      protected override void OnResume()
      {
         this.UserName = this.cache.FromCache<string>("UserName");
         this.NickName = this.cache.FromCache<string>("NickName");
         //Set the focus back to proper control
         //string focusedElement = this.cache.FromCache<string>("FocusedElement");
         //this.SetFocusedElement(focusedElement);
      }

      private string GetFocusedElement()
      {
         QueryActiveControlMessage queryActiveControlMessage = new QueryActiveControlMessage() { IsReadMode = true };
         this.Messenger.Send(queryActiveControlMessage);
         return queryActiveControlMessage.Name;
      }

      private void SetFocusedElement(string focusedElementName)
      {
         QueryActiveControlMessage queryActiveControlMessage = new QueryActiveControlMessage() { Name = focusedElementName };
         this.Messenger.Send(queryActiveControlMessage);
      }


      protected override void OnPageLoaded()
      {
         if (this.IsResuming)
         {
            string focusedElement = this.cache.FromCache<string>("FocusedElement");
            this.SetFocusedElement(focusedElement);
         }
      }
   }
}
